@@ -15,54 +15,36 @@
 #ifndef __H5Location_H
 #define __H5Location_H
 
-#include "H5Classes.h"          // constains forward class declarations
+#include "H5Classes.h"        // constains forward class declarations
 
 namespace H5 {
 
-class H5Location;  // forward declaration for UserData4Aiterate
-
-// Define the operator function pointer for H5Aiterate().
-typedef void (*attr_operator_t)(H5Location& loc/*in*/,
-                                 const H5std_string attr_name/*in*/,
-                                 void *operator_data/*in,out*/);
-
-class UserData4Aiterate { // user data for attribute iteration
-   public:
-        attr_operator_t op;
-        void* opData;
-        H5Location* location;
-};
-
 /*! \class H5Location
-    \brief H5Location is an abstract base class, providing a collection of
-    wrappers of the C functions that take a location identifier, which can be
-    either file, group, dataset, attribute, or named datatype.
+    \brief H5Location is an abstract base class, added in version 1.8.12.
+
+    It provides a collection of wrappers for the C functions that take a
+    location identifier to specify the HDF5 object.  The location identifier
+    can be either file, group, dataset, attribute, or named datatype.
+    Wrappers for H5A functions stay in H5Object.
 */
-//  Inheritance: IdComponent
+// Inheritance: IdComponent
 class H5_DLLCPP H5Location : public IdComponent {
    public:
-        // Determines the number of attributes belong to this object.
-        int getNumAttrs() const;
-
-        // Checks if a link of a given name exists in this location
+        // Checks if a link of a given name exists in a location
         bool nameExists(const char* name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
         bool nameExists(const H5std_string& name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
+        // Checks if a link of a given name exists in a location
+        // Deprecated in favor of nameExists for better name.
+        bool exists(const char* name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        bool exists(const H5std_string& name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+
         // Flushes all buffers associated with this location to disk.
         void flush(H5F_scope_t scope) const;
 
         // Gets the name of the file, specified by this location.
         H5std_string getFileName() const;
 
-        // Retrieves information about an object at this location
-            // specified by location
-        void getObjectInfo(H5O_info_t *oinfo) const;
-            // specified by the object's name
-        void getObjectInfo(const char *name, H5O_info_t *oinfo,
-                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
-        void getObjectInfo(const H5std_string& name, H5O_info_t *oinfo,
-                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
-
 #ifndef H5_NO_DEPRECATED_SYMBOLS
         // Retrieves the type of object that an object reference points to.
         H5G_obj_t getObjType(void *ref, H5R_type_t ref_type = H5R_OBJECT) const;
@@ -73,9 +55,6 @@ class H5_DLLCPP H5Location : public IdComponent {
         // Note: getRefObjType deprecates getObjType, but getObjType's name is
         // misleading, so getRefObjType is used in the new function instead.
 
-        // Returns the object header version of an object
-        unsigned objVersion() const;
-
         // Sets the comment for an HDF5 object specified by its name.
         void setComment(const char* name, const char* comment) const;
         void setComment(const H5std_string& name, const H5std_string& comment) const;
@@ -95,7 +74,7 @@ class H5_DLLCPP H5Location : public IdComponent {
         // in this object.
         void reference(void* ref, const char* name, 
                         H5R_type_t ref_type = H5R_OBJECT) const;
-        void reference(void* ref, const H5std_string& name, 
+        void reference(void* ref, const H5std_string& name,
                         H5R_type_t ref_type = H5R_OBJECT) const;
         void reference(void* ref, const char* name, const DataSpace& dataspace,
                         H5R_type_t ref_type = H5R_DATASET_REGION) const;
@@ -104,18 +83,130 @@ class H5_DLLCPP H5Location : public IdComponent {
 
         // Open a referenced object whose location is specified by either
         // a file, an HDF5 object, or an attribute.
-        void dereference(const H5Location& loc, const void* ref, H5R_type_t ref_type = H5R_OBJECT);
-        void dereference(const Attribute& attr, const void* ref, H5R_type_t ref_type = H5R_OBJECT);
+        void dereference(const H5Location& loc, const void* ref, H5R_type_t ref_type = H5R_OBJECT, const PropList& plist = PropList::DEFAULT);
+        // Removed in 1.10.1, because H5Location is baseclass
+        //void dereference(const Attribute& attr, const void* ref, H5R_type_t ref_type = H5R_OBJECT, const PropList& plist = PropList::DEFAULT);
 
         // Retrieves a dataspace with the region pointed to selected.
         DataSpace getRegion(void *ref, H5R_type_t ref_type = H5R_DATASET_REGION) const;
 
-        // Opens an object at this location, without knowing the object type.
-        hid_t openObjId(const char* name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
-        hid_t openObjId(const H5std_string& name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        // Create a new group with using link create property list.
+        Group createGroup(const char* name, const LinkCreatPropList& lcpl) const;
+        Group createGroup(const H5std_string& name, const LinkCreatPropList& lcpl) const;
 
-        // Closes an object opened by openObjId()
-        static void closeObjId(hid_t obj_id);
+// From CommonFG
+        // Creates a new group at this location which can be a file
+        // or another group.
+        Group createGroup(const char* name, size_t size_hint = 0) const;
+        Group createGroup(const H5std_string& name, size_t size_hint = 0) const;
+
+        // Opens an existing group in a location which can be a file
+        // or another group.
+        Group openGroup(const char* name) const;
+        Group openGroup(const H5std_string& name) const;
+
+        // Creates a new dataset in this location.
+        DataSet createDataSet(const char* name, const DataType& data_type, const DataSpace& data_space, const DSetCreatPropList& create_plist = DSetCreatPropList::DEFAULT, const DSetAccPropList& dapl = DSetAccPropList::DEFAULT, const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT) const;
+        DataSet createDataSet(const H5std_string& name, const DataType& data_type, const DataSpace& data_space, const DSetCreatPropList& create_plist = DSetCreatPropList::DEFAULT, const DSetAccPropList& dapl = DSetAccPropList::DEFAULT, const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT) const;
+
+        // Deprecated to add LinkCreatPropList and DSetAccPropList - 1.10.3
+        // DataSet createDataSet(const char* name, const DataType& data_type, const DataSpace& data_space, const DSetCreatPropList& create_plist = DSetCreatPropList::DEFAULT) const;
+        // DataSet createDataSet(const H5std_string& name, const DataType& data_type, const DataSpace& data_space, const DSetCreatPropList& create_plist = DSetCreatPropList::DEFAULT) const;
+
+        // Opens an existing dataset at this location.
+        // DSetAccPropList is added - 1.10.3
+        DataSet openDataSet(const char* name, const DSetAccPropList& dapl = DSetAccPropList::DEFAULT) const;
+        DataSet openDataSet(const H5std_string& name, const DSetAccPropList& dapl = DSetAccPropList::DEFAULT) const;
+
+        H5L_info2_t getLinkInfo(const char* link_name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        H5L_info2_t getLinkInfo(const H5std_string& link_name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+
+        // Returns the value of a symbolic link.
+        H5std_string getLinkval(const char* link_name, size_t size=0) const;
+        H5std_string getLinkval(const H5std_string& link_name, size_t size=0) const;
+
+        // Returns the number of objects in this group.
+        // Deprecated - moved to H5::Group in 1.10.2.
+        hsize_t getNumObjs() const;
+
+        // Retrieves the name of an object in this group, given the
+        // object's index.
+        H5std_string getObjnameByIdx(hsize_t idx) const;
+        ssize_t getObjnameByIdx(hsize_t idx, char* name, size_t size) const;
+        ssize_t getObjnameByIdx(hsize_t idx, H5std_string& name, size_t size) const;
+
+        // Retrieves the type of an object in this file or group, given the
+        // object's name
+        H5O_type_t childObjType(const H5std_string& objname) const;
+        H5O_type_t childObjType(const char* objname) const;
+        H5O_type_t childObjType(hsize_t index, H5_index_t index_type=H5_INDEX_NAME, H5_iter_order_t order=H5_ITER_INC, const char* objname=".") const;
+
+        // Returns the object header version of an object in this file or group,
+        // given the object's name.
+        unsigned childObjVersion(const char* objname) const;
+        unsigned childObjVersion(const H5std_string& objname) const;
+
+        // Retrieves information about an HDF5 object.
+        void getObjinfo(H5O_info2_t& objinfo, unsigned fields = H5O_INFO_BASIC) const;
+
+        // Retrieves information about an HDF5 object, given its name.
+        void getObjinfo(const char* name, H5O_info2_t& objinfo,
+                unsigned fields = H5O_INFO_BASIC,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        void getObjinfo(const H5std_string& name, H5O_info2_t& objinfo,
+                unsigned fields = H5O_INFO_BASIC,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+
+        // Retrieves information about an HDF5 object, given its index.
+        void getObjinfo(const char* grp_name, H5_index_t idx_type,
+                H5_iter_order_t order, hsize_t idx, H5O_info2_t& objinfo,
+                unsigned fields = H5O_INFO_BASIC,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        void getObjinfo(const H5std_string& grp_name, H5_index_t idx_type,
+                H5_iter_order_t order, hsize_t idx, H5O_info2_t& objinfo,
+                unsigned fields = H5O_INFO_BASIC,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+
+        // Retrieves native native information about an HDF5 object.
+        void getNativeObjinfo(H5O_native_info_t& objinfo, unsigned fields = H5O_NATIVE_INFO_HDR) const;
+
+        // Retrieves native information about an HDF5 object, given its name.
+        void getNativeObjinfo(const char* name, H5O_native_info_t& objinfo,
+                unsigned fields = H5O_NATIVE_INFO_HDR,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        void getNativeObjinfo(const H5std_string& name, H5O_native_info_t& objinfo,
+                unsigned fields = H5O_NATIVE_INFO_HDR,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+
+        // Retrieves native information about an HDF5 object, given its index.
+        void getNativeObjinfo(const char* grp_name, H5_index_t idx_type,
+                H5_iter_order_t order, hsize_t idx, H5O_native_info_t& objinfo,
+                unsigned fields = H5O_NATIVE_INFO_HDR,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        void getNativeObjinfo(const H5std_string& grp_name, H5_index_t idx_type,
+                H5_iter_order_t order, hsize_t idx, H5O_native_info_t& objinfo,
+                unsigned fields = H5O_NATIVE_INFO_HDR,
+                const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+
+#ifndef H5_NO_DEPRECATED_SYMBOLS
+        // Returns the type of an object in this group, given the
+        // object's index.
+        H5G_obj_t getObjTypeByIdx(hsize_t idx) const;
+        H5G_obj_t getObjTypeByIdx(hsize_t idx, char* type_name) const;
+        H5G_obj_t getObjTypeByIdx(hsize_t idx, H5std_string& type_name) const;
+
+        // Returns information about an HDF5 object, given by its name,
+        // at this location. - Deprecated
+        void getObjinfo(const char* name, hbool_t follow_link, H5G_stat_t& statbuf) const;
+        void getObjinfo(const H5std_string& name, hbool_t follow_link, H5G_stat_t& statbuf) const;
+        void getObjinfo(const char* name, H5G_stat_t& statbuf) const;
+        void getObjinfo(const H5std_string& name, H5G_stat_t& statbuf) const;
+
+        // Iterates over the elements of this group - not implemented in
+        // C++ style yet.
+        int iterateElems(const char* name, int *idx, H5G_iterate_t op, void *op_data);
+        int iterateElems(const H5std_string& name, int *idx, H5G_iterate_t op, void *op_data);
+#endif /* H5_NO_DEPRECATED_SYMBOLS */
 
         // Creates a soft link from link_name to target_name.
         void link(const char *target_name, const char *link_name,
@@ -128,11 +219,11 @@ class H5_DLLCPP H5Location : public IdComponent {
 
         // Creates a hard link from new_name to curr_name.
         void link(const char *curr_name,
-             const H5Location& new_loc, const char *new_name,
+             const Group& new_loc, const char *new_name,
              const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
              const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
         void link(const H5std_string& curr_name,
-             const H5Location& new_loc, const H5std_string& new_name,
+             const Group& new_loc, const H5std_string& new_name,
              const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
              const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
@@ -146,23 +237,37 @@ class H5_DLLCPP H5Location : public IdComponent {
              const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
              const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
+        // Creates a link of the specified type from new_name to current_name;
+        // both names are interpreted relative to the specified location id.
+        // Deprecated due to inadequate functionality.
+        void link(H5L_type_t link_type, const char* curr_name, const char* new_name) const;
+        void link(H5L_type_t link_type, const H5std_string& curr_name, const H5std_string& new_name) const;
+
         // Removes the specified link from this location.
         void unlink(const char *link_name,
             const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
         void unlink(const H5std_string& link_name,
             const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
-        // Copies a link from this location to another.
+        // Mounts the file 'child' onto this location.
+        void mount(const char* name, const H5File& child, const PropList& plist) const;
+        void mount(const H5std_string& name, const H5File& child, const PropList& plist) const;
+
+        // Unmounts the file named 'name' from this parent location.
+        void unmount(const char* name) const;
+        void unmount(const H5std_string& name) const;
+
+        // Copies a link from a group to another.
         void copyLink(const char *src_name,
-             const H5Location& dst, const char *dst_name,
+             const Group& dst, const char *dst_name,
              const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
              const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
         void copyLink(const H5std_string& src_name,
-             const H5Location& dst, const H5std_string& dst_name,
+             const Group& dst, const H5std_string& dst_name,
              const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
              const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
-        // Makes a copy of a link in the same location.
+        // Makes a copy of a link in the same group.
         void copyLink(const char *src_name, const char *dst_name,
              const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
              const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
@@ -171,17 +276,17 @@ class H5_DLLCPP H5Location : public IdComponent {
              const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
              const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
-        // Renames a link in this location and moves to a new location.
+        // Renames a link in this group and moves to a new location.
         void moveLink(const char* src_name,
-            const H5Location& dst, const char* dst_name,
+            const Group& dst, const char* dst_name,
             const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
             const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
         void moveLink(const H5std_string& src_name,
-            const H5Location& dst, const H5std_string& dst_name,
+            const Group& dst, const H5std_string& dst_name,
             const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
             const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
-        // Renames a link in this location.
+        // Renames a link in this group.
         void moveLink(const char* src_name, const char* dst_name,
             const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
             const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
@@ -190,68 +295,21 @@ class H5_DLLCPP H5Location : public IdComponent {
             const LinkCreatPropList& lcpl = LinkCreatPropList::DEFAULT,
             const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
 
-        // Returns the information of the named link.
-        H5L_info_t getLinkInfo(const char* link_name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
-        H5L_info_t getLinkInfo(const H5std_string& link_name, const LinkAccPropList& lapl = LinkAccPropList::DEFAULT) const;
+        // Renames an object at this location.
+        // Deprecated due to inadequate functionality.
+        void move(const char* src, const char* dst) const;
+        void move(const H5std_string& src, const H5std_string& dst) const;
 
-        // Returns the value of a symbolic link.
-        H5std_string getLinkval(const char* link_name, size_t size=0) const;
-        H5std_string getLinkval(const H5std_string& link_name, size_t size=0) const;
+// end From CommonFG
 
-        ///\brief Returns an identifier. (pure virtual)
-        virtual hid_t getId() const = 0;
+        /// For subclasses, H5File and Group, to throw appropriate exception.
+        virtual void throwException(const H5std_string& func_name, const H5std_string& msg) const;
 
-/***************************************************************************
-                           Notes for H5A wrappers
-                           ======================
-        These H5A wrappers are marked "deprecated" in 1.8.19.
-        They are moved to H5Object to prevent the object id from being
-        passed in to H5A APIs.
-        Updated: they are removed from source code in 1.8.21.
-***************************************************************************/
-
-        // Creates an attribute for the specified object at this location
-        // PropList is currently not used, so always be default.
-        // Deprecated
-        //virtual Attribute createAttribute(const char* name, const DataType& type, const DataSpace& space, const PropList& create_plist = PropList::DEFAULT) const;
-        //virtual Attribute createAttribute(const H5std_string& name, const DataType& type, const DataSpace& space, const PropList& create_plist = PropList::DEFAULT) const;
-
-        // Given its name, opens the attribute that belongs to an object at
-        // this location.
-        // Deprecated
-        //virtual Attribute openAttribute(const char* name) const;
-        //virtual Attribute openAttribute(const H5std_string& name) const;
-
-        // Given its index, opens the attribute that belongs to an object at
-        // this location.
-        //virtual Attribute openAttribute(const unsigned int idx) const; // Deprecated
-
-        // Iterate user's function over the attributes at this location.
-        virtual int iterateAttrs(attr_operator_t user_op, unsigned* idx = NULL,
-                                 void* op_data = NULL); // Deprecated
-
-        // Checks whether the named attribute exists at this location.
-        // Deprecated
-        //virtual bool attrExists(const char* name) const;
-        //virtual bool attrExists(const H5std_string& name) const;
-
-        // Renames the named attribute to a new name.
-        // Deprecated
-        //virtual void renameAttr(const char* oldname, const char* newname) const;
-        //virtual void renameAttr(const H5std_string& oldname, const H5std_string& newname) const;
-
-        // Removes the named attribute from this location.
-        // Deprecated
-        //virtual void removeAttr(const char* name) const;
-        //virtual void removeAttr(const H5std_string& name) const;
-
-/**************************** End of H5A note *******************************/
-
-   protected:
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
         // Default constructor
         H5Location();
 
+   protected:
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
         // *** Deprecation warning ***
         // The following two constructors are no longer appropriate after the
         // data member "id" had been moved to the sub-classes.
@@ -260,13 +318,13 @@ class H5_DLLCPP H5Location : public IdComponent {
         // removal does not raise any problems in two 1.10 releases.
 
         // Creates a copy of an existing object giving the location id.
-        H5Location(const hid_t loc_id);
+        // H5Location(const hid_t loc_id);
 
         // Creates a reference to an HDF5 object or a dataset region.
         void p_reference(void* ref, const char* name, hid_t space_id, H5R_type_t ref_type) const;
 
         // Dereferences a ref into an HDF5 id.
-        hid_t p_dereference(hid_t loc_id, const void* ref, H5R_type_t ref_type, const char* from_func);
+        hid_t p_dereference(hid_t loc_id, const void* ref, H5R_type_t ref_type, const PropList& plist, const char* from_func);
 
 #ifndef H5_NO_DEPRECATED_SYMBOLS
         // Retrieves the type of object that an object reference points to.
@@ -278,7 +336,7 @@ class H5_DLLCPP H5Location : public IdComponent {
 
         // Sets the identifier of this object to a new value. - this one
         // doesn't increment reference count
-        virtual void p_setId(const hid_t new_id) = 0;
+        //virtual void p_setId(const hid_t new_id);
 
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
@@ -289,13 +347,3 @@ class H5_DLLCPP H5Location : public IdComponent {
 } // namespace H5
 
 #endif // __H5Location_H
-
-// Modification
-//      Oct 1, 2013 -BMR
-//          H5Location is added in version 1.8.12.
-//          Most of these methods were in H5Object but are now moved here
-//          because a location can be a file, group, dataset, or named datatype.
-//      May 04, 2017 -BMR
-//          Wrappers for H5A functions are copied to H5Object because H5A
-//          functions do not take an attribute id as loc_id.  The original
-//          wrappers will be deprecated in future releases.
