@@ -25,7 +25,8 @@
 #include "H5LTpublic.h" /* lite */
 #include "H5IMpublic.h" /* image */
 #include "H5TBpublic.h" /* table */
-#include "H5PTpublic.h" /* table */
+#include "H5PTpublic.h" /* packet table */
+#include "H5LDpublic.h" /* lite dataset */
 
 #endif /*H5_INCLUDE_HL*/
 
